from .utils import load_json

class Conditions:
    def _get_conditions(self, service_key):
        """Load conditions from a JSON file."""
        return load_json("conditionKeys",f"{service_key.lower()}.json")

    def get_condition_keys_for_service(self, service_key):
        """Returns a list of all condition keys for a given service key."""
        return [key['key'] for key in self._get_conditions(service_key).values()]

    def get_condition_key_details(self, service_key, condition_key):
        """Returns the details of a specific condition key."""
        data = self._get_conditions(service_key)
        lower_condition_key = condition_key.lower()
        if lower_condition_key in data:
            return data[lower_condition_key]
        raise Exception(f"Condition key '{condition_key}' not found for service '{service_key}'")

    def condition_key_exists(self, service_key, condition_key):
        """Checks if a condition key exists for a given service key."""
        data = self._get_conditions(service_key)
        lower_condition_key = condition_key.lower()
        return lower_condition_key in data


from .utils import load_json

class Services:

    def get_service_keys(self):
        """Returns a list of all service keys."""
        return load_json('services.json')

    def get_service_name(self, service_key):
        """Get the name of a service by its key."""
        data = load_json('serviceNames.json')
        lower_key = service_key.lower()
        if(lower_key in data):
            return data[lower_key]
        raise Exception(f"Service key {service_key} found")

    def service_exists(self, service_key):
        """Checks if a service key exists."""
        data = load_json('serviceNames.json')
        lower_key = service_key.lower()
        return lower_key in data


from .utils import load_json

class Actions:
    def _get_actions(self, service_key):
        """Load actions from a JSON file."""
        return load_json("actions",f"{service_key.lower()}.json")

    def get_actions_for_service(self, service_key):
        """Returns a list of all actions for a given service key."""
        return [action['name'] for action in self._get_actions(service_key).values()]

    def get_action_details(self, service_key, action_key):
        """Returns the details of a specific action."""
        lower_action = action_key.lower()
        data = self._get_actions(service_key)
        if lower_action in data:
            return data[lower_action]
        raise Exception(f"Action '{action_key}' not found for service '{service_key}'")

    def action_exists(self, service_key, action_key):
        """Checks if an action exists for a given service key."""
        lower_action = action_key.lower()
        return lower_action in self._get_actions(service_key)


import unittest

from iamdata.services import Services

class ServicesTest(unittest.TestCase):
    def test_get_service_keys(self):
        services = Services()
        result = services.get_service_keys()
        self.assertIsInstance(result, list)

    def test_get_service_name_exists(self):
        services = Services()
        result = services.get_service_name("s3")
        self.assertEqual(result, "Amazon S3")

    def test_get_service_name_not_exists(self):
        services = Services()
        with self.assertRaises(Exception) as context:
            services.get_service_name("nonexistent_service")
        self.assertEqual(str(context.exception), "Service key nonexistent_service found")

    def test_service_exists(self):
        services = Services()
        result = services.service_exists("s3")
        self.assertTrue(result)

    def test_service_not_exists(self):
        services = Services()
        result = services.service_exists("nonexistent_service")
        self.assertFalse(result)


# iam_data_lib/resources.py
from .utils import load_json

class Resources:

    def _get_resources(self, service_key):
        """Load resources from a JSON file."""
        return load_json("resourceTypes",f"{service_key.lower()}.json")

    def get_resource_types_for_service(self, service_key):
        """Returns a list of all resource types for a given service key."""
        return [r['key'] for r in self._get_resources(service_key).values()]

    def get_resource_type_details(self, service_key, resource_type_key):
        """Returns the details of a specific resource type."""
        data = self._get_resources(service_key)
        lower_resource_type_key = resource_type_key.lower()
        if lower_resource_type_key in data:
            return data[lower_resource_type_key]
        raise Exception(f"Resource type '{resource_type_key}' not found for service '{service_key}'")

    def resource_type_exists(self, service_key, resource_type_key):
        """Checks if a resource type exists for a given service key."""
        lower_resource_type_key = resource_type_key.lower()
        return lower_resource_type_key in self._get_resources(service_key)


import unittest

from iamdata.actions import Actions

class ActionsTest(unittest.TestCase):
    def test_get_actions_for_service(self):
        actions = Actions()
        result = actions.get_actions_for_service("s3")
        self.assertIsInstance(result, list)

    def test_get_action_details_exists(self):
        actions = Actions()
        result = actions.get_action_details("s3", "GetObject")
        self.assertIsInstance(result, dict)

    def test_get_action_details_not_exists(self):
        actions = Actions()
        with self.assertRaises(Exception) as context:
            actions.get_action_details("s3", "nonexistent_action")
        self.assertEqual(str(context.exception), "Action 'nonexistent_action' not found for service 's3'")

    def test_action_exists(self):
        actions = Actions()
        result = actions.action_exists("s3", "GetObject")
        self.assertTrue(result)

    def test_action_not_exists(self):
        actions = Actions()
        result = actions.action_exists("s3", "nonexistent_action")
        self.assertFalse(result)


from tomlkit import load, dumps
import sys

doc = load(open("pyproject.toml","r"))
doc["project"]["version"]
doc["project"]["version"] = sys.argv[1]
with open("pyproject.toml", "w") as f:
    f.write(dumps(doc))


import unittest

from iamdata.conditions import Conditions

class ConditionsTest(unittest.TestCase):
    def test_get_condition_keys_for_service(self):
        conditions = Conditions()
        result = conditions.get_condition_keys_for_service("s3")
        self.assertIsInstance(result, list)

    def test_get_condition_key_details(self):
        conditions = Conditions()
        result = conditions.get_condition_key_details("s3", "s3:AccessGrantsInstanceArn")
        self.assertIsInstance(result, dict)

    def test_condition_key_exists(self):
        conditions = Conditions()
        result = conditions.condition_key_exists("s3", "s3:AccessGrantsInstanceArn")
        self.assertTrue(result)

    def test_condition_key_not_exists(self):
        conditions = Conditions()
        result = conditions.condition_key_exists("s3", "nonexistent_condition_key")
        self.assertFalse(result)

    def test_get_condition_key_details_not_exists(self):
        conditions = Conditions()
        with self.assertRaises(Exception) as context:
            conditions.get_condition_key_details("s3", "nonexistent_condition_key")
        self.assertEqual(str(context.exception), "Condition key 'nonexistent_condition_key' not found for service 's3'")


import json
import os

current_dir = os.path.dirname(__file__)
data_dir = os.path.join(current_dir, 'data')

_cache = {}

def load_json(*file_path_parts):
    file_path = os.path.join(data_dir, *file_path_parts)
    if file_path in _cache:
        return _cache[file_path]
    with open(file_path, 'r') as f:
        content = json.load(f)
        _cache[file_path] = content
        return content


import unittest

from iamdata.resources import Resources

class ResourcesTest(unittest.TestCase):
    def test_get_resource_types_for_service(self):
        resources = Resources()
        result = resources.get_resource_types_for_service("s3")
        self.assertIsInstance(result, list)

    def test_get_resource_type_details(self):
        resources = Resources()
        result = resources.get_resource_type_details("s3", "Bucket")
        self.assertIsInstance(result, dict)

    def test_get_resource_type_details_not_exists(self):
        resources = Resources()
        with self.assertRaises(Exception) as context:
            resources.get_resource_type_details("s3", "nonexistent_resource")
        self.assertEqual(str(context.exception), "Resource type 'nonexistent_resource' not found for service 's3'")

    def test_resource_type_exists(self):
        resources = Resources()
        result = resources.resource_type_exists("s3", "Bucket")
        self.assertTrue(result)

    def test_resource_type_not_exists(self):
        resources = Resources()
        result = resources.resource_type_exists("s3", "nonexistent_resource")
        self.assertFalse(result)


# iam_data_lib/__init__.py
from .utils import load_json
from .services import Services
from .actions import Actions
from .resources import Resources
from .conditions import Conditions

from datetime import datetime

class IAMData:
    def __init__(self):
        self.services = Services()
        self.actions = Actions()
        self.resources = Resources()
        self.conditions = Conditions()

    def _get_metadata(self):
        """Load metadata from a JSON file."""
        return load_json("metadata.json")

    def data_version(self):
        """Returns the version of the IAM data."""
        return self._get_metadata()['version']

    def data_updated_at(self):
        """Returns the last updated timestamp of the IAM data."""
        return datetime.strptime(self._get_metadata()['updatedAt'], '%Y-%m-%dT%H:%M:%S.%fZ')